/*
 * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */
package com.openexchange.usm.connector.commands;

import static com.openexchange.usm.connector.commands.CommandConstants.FOLDERID;
import static com.openexchange.usm.connector.commands.CommandConstants.SESSIONID;
import static com.openexchange.usm.connector.commands.CommandConstants.SYNCID;
import javax.servlet.http.HttpServletRequest;
import org.json.JSONArray;
import org.json.JSONException;
import com.openexchange.usm.api.exceptions.USMException;
import com.openexchange.usm.api.session.DataObject;
import com.openexchange.usm.api.session.Folder;
import com.openexchange.usm.json.ConnectorBundleErrorCodes;
import com.openexchange.usm.json.USMJSONAPIException;
import com.openexchange.usm.json.USMJSONServlet;
import com.openexchange.usm.json.response.ResponseObject;
import com.openexchange.usm.json.response.ResponseStatusCode;

/**
 * Handler for the GetCachedData USM-JSON-Command.
 * @author afe
 *
 */
public class GetCachedDataHandler extends NormalCommandHandler {
	private static final String[] REQUIRED_PARAMETERS = { SESSIONID, SYNCID };
	private static final String[] OPTIONAL_PARAMETERS = { FOLDERID };

    public GetCachedDataHandler(USMJSONServlet servlet, HttpServletRequest request) throws USMJSONAPIException {
        super(servlet, request);
    }

	@Override
	public ResponseObject handleRequest() throws USMJSONAPIException {
		String folderID = getStringParameter(FOLDERID, null);
		long syncId = getSyncID();
		DataObject[] resultElements = getCachedElements(syncId, folderID);
		if (resultElements == null) {
            throw new USMJSONAPIException(ConnectorBundleErrorCodes.SYNC_UPDATE_INVALID_SYNCID,
                ResponseStatusCode.UNKNOWN_SYNCID, "Unknown SyncID "+ syncId + " for " + ((folderID == null) ? "folder hierarchy" : folderID));
        }
		_allFolders = resultElements;
		JSONArray data = createResponseData(resultElements);
		return new ResponseObject(ResponseStatusCode.SUCCESS, data);
	}

	private JSONArray createResponseData(DataObject[] resultElements) throws USMJSONAPIException {
		JSONArray data = new JSONArray();
		if (resultElements != null) {
			for (DataObject dataObject : resultElements) {
				try {
					data.put(storeCompleteDataObjectInJSONObject(dataObject));
				} catch (JSONException e) {
					throw USMJSONAPIException.createJSONError(ConnectorBundleErrorCodes.GET_CACHED_DATA_JSON_ERROR, e);
				}
			}
		}
		return data;
	}

	private DataObject[] getCachedElements(long syncID, String folderID) throws USMJSONAPIException {
		try {
			if (folderID == null) {
				//get cached folder hierarchy
				return _session.getCachedFolders(syncID);
			}
			Folder folder = getFolderByUUID(folderID);
			return _session.getCachedFolderElements(folder.getID(), folder.getElementsContentType(), syncID);
		} catch (USMJSONAPIException e) {
			throw e;
		} catch (USMException e) {
			throw USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.GET_CACHED_DATA_GET_ELEMENTS_ERROR,
					e);
		}
	}

	@Override
	protected String[] getOptionalParameters() {
		return OPTIONAL_PARAMETERS;
	}

	@Override
	protected String[] getRequiredParameters() {
		return REQUIRED_PARAMETERS;
	}
}
